/*******************************************************************************
* SAT4J: a SATisfiability library for Java Copyright (C) 2004-2008 Daniel Le Berre
*
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Alternatively, the contents of this file may be used under the terms of
* either the GNU Lesser General Public License Version 2.1 or later (the
* "LGPL"), in which case the provisions of the LGPL are applicable instead
* of those above. If you wish to allow use of your version of this file only
* under the terms of the LGPL, and not to allow others to use your version of
* this file under the terms of the EPL, indicate your decision by deleting
* the provisions above and replace them with the notice and other provisions
* required by the LGPL. If you do not delete the provisions above, a recipient
* may use your version of this file under the terms of the EPL or the LGPL.
* 
* Based on the original MiniSat specification from:
* 
* An extensible SAT solver. Niklas Een and Niklas Sorensson. Proceedings of the
* Sixth International Conference on Theory and Applications of Satisfiability
* Testing, LNCS 2919, pp 502-518, 2003.
*
* See www.minisat.se for the original solver in C++.
* 
*******************************************************************************/
package org.sat4j.minisat.core;

import org.sat4j.specs.IConstr;

/**
 * An assertingClauseGenerator is responsible for the creation of an asserting
 * clause during conflict analysis. An asserting clause is a clause that will
 * become unit when the solver will backtrack to the latest decision level,
 * providing a nice way for the solver to backtrack.
 * 
 * @author leberre
 */
public interface AssertingClauseGenerator {

    /**
     * hook method called before the analysis.
     */
    void initAnalyze();

    /**
     * hook method called when a literal from the current decision lelvel is
     * found.
     * 
     * @param p
     *            the literal in the current decision level
     */
    void onCurrentDecisionLevelLiteral(int p);

    /**
     * method indicating if an asserting clause has been built. note that this
     * method is called right after a resolution step is finished.
     * 
     * @param reason
     *            the reason of the current literal assignment
     * @return false iff the current clause is assertive
     */
    boolean clauseNonAssertive(IConstr reason);
}
